/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Cocoa/Cocoa.h>
#import "IDLDefinitionScope.h"
#import "IDLDefinition_Private.h"
#import "ObjectCell.h"
#import "IDL.h"
#import "IDLLink.h"
#import "IDLReference.h"
#import "IDLNative.h"
#import "IDLTypedValue.h"
#import "IDLViewer.h"

@implementation IDLDefinition(Viewer)

static BOOL sortMatrixItems = YES;
static int sortIfGreaterThan = 16;

NSString* IDLDefinitionPboardType = @"IDLDefinitionPboardType";

+ (void) _registerDefaults
{
    NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithBool:sortMatrixItems],@"sortMatrixItems",
        [NSNumber numberWithInt:sortIfGreaterThan],@"sortIfGreaterThan",
        nil];
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    [defaults registerDefaults:dict];
    sortMatrixItems = [defaults boolForKey:@"sortMatrixItems"];
    sortIfGreaterThan = [defaults integerForKey:@"sortIfGreaterThan"];
}

- (BOOL) sortMatrixItems
{
    return sortMatrixItems;
}

- (int) sortIfGreaterThan
{
    return sortIfGreaterThan;
}

+ (BOOL) sortMatrixItems
{
    return sortMatrixItems;
}

+ (int) sortIfGreaterThan
{
    return sortIfGreaterThan;
}

+ (void) setSortMatrixItems:(BOOL)flag
{
    sortMatrixItems = flag?YES:NO;
}

+ (void) setSortIfGreaterThan:(int)n
{
    sortIfGreaterThan = n;
}

- (NSString*) category
{
    return @"- Types -";
}

- (NSString*)stringValue
{
    return @"definition";
}

- (NSString*) title
{
    return nil;
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"primitive13"];
}

- (void) loadCell:cell
{
    [cell setRepresentedObject:self];
    [cell setLoaded:YES];
	[cell setImage:[self cellImage]];
    [cell setLeaf:YES];
}

- (void) fillMatrix:matrix
{
}

- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard
{
    [pboard declareTypes:[NSArray arrayWithObjects:IDLDefinitionPboardType, NSStringPboardType, nil] owner:self];
    [pboard setString:[self typeId] forType:IDLDefinitionPboardType];
    [pboard setString:[self typeId] forType:NSStringPboardType];
    return YES;
}

@end

@implementation IDLPrimitive(Viewer)

- (NSString*) title
{
    return @"";
}

- (NSString*) stringValue
{
    return [self name];
}

- (NSString*) view:(NSView*)view stringForToolTip:(NSToolTipTag)tag point:(NSPoint)p userData:(void*)data
{
    return [self stringValue];
}

@end

@implementation IDLLink(Viewer)

- (NSString*) category
{
    return [_link category];
}

@end

@implementation IDLReference(Viewer)

- (NSString*) stringValue
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    if(def)
        return [def stringValue];
    else
        return @"<undefined>";
}

- (NSImage*) cellImage
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    if(def)
		return [def cellImage];
	else
		return nil;
}

- (void) loadCell:cell
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    if(def)
        [def loadCell:cell];
    else{
        [cell setRepresentedObject:self];
        [cell setLoaded:YES];
        [cell setLeaf:YES];
    }
}

- (void) fillMatrix:matrix
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    if(def)
        [def fillMatrix:matrix];
}

- (NSString*) title
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    return def?[def title] : @"<undefined>";
}

- (NSString*) category
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    if(def)
        return [def category];
    else
        return @"- Types -";
}

@end

@implementation IDLNative(Viewer)

- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard
{
    return NO;
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"primitive13"];
}
- (void) loadCell:cell
{
    [cell setRepresentedObject:self];
    [cell setLoaded:YES];
    [cell setLeaf:YES];
	[cell setImage:[self cellImage]];
}

- (NSString*)stringValue
{
    if([self className])
        return [NSString stringWithFormat:@"native %@ <%@>", [self name], [self className]];
    else
   	return [NSString stringWithFormat:@"native %@",[self name]];
}

@end

@implementation IDLTypedValue(Viewer)

- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard
{
    return NO;
}

- (NSString*) category
{
    if(!_isHidden)
        return @"- Contants -";
    else
        return nil;
}

- (NSString*) stringValue
{
    return [NSString stringWithFormat:@"const %@ = %@", 
        [_type name], 
        _value?[_type valueDescriptionOf:_value]:@"<undefined>"];
}

- (NSString*) title
{
    return @"const";
}

@end

