/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <AppKit/AppKit.h>
#import "StockAdmin.h"
#import <ADORB/ADORB.h>
#import "QuoteItem.h"
#import "StockMarket.h"
#import "StockAdminProto.h"

@implementation StockAdmin

- initWithController:ctrl
{
    self = [super init];
	_controller = ctrl;
	@try{
		_server = [[(id)[_controller server]getStockAdmin] retain];
		[_server setDelegate:self];
	}@catch(NSException *localException){
		NSLog(@"%@, %@", [localException name], [localException reason]);
	}
	[_controller addObserver:self forKeyPath:@"server" options:0 context:NULL];
    if([NSBundle loadNibNamed:@"StockAdmin" owner:self]){
		_item = [[QuoteItem alloc]init];
        return self;
    }else{
        [self release];
        return nil;
    }
}

- (void) dealloc
{
	[_controller removeObserver:self forKeyPath:@"server"];
	[window release];
    [_server release];
	[_item release];
    [super dealloc];
}

- window
{
    return window;
}

- (void) addSymbol:sender
{
    @try{
		[_server addQuote:_item];
    }@catch(NSException *localException){
		NSRunAlertPanel(@"Exception", @"%@\n%@",nil,nil,nil,[localException name], [localException reason]);
    }
}

- (void) objectDidInvalidate:(NSNotification*)notification
{
	if([notification object] == _server){
		[self willChangeValueForKey:@"server"];
		[_server release];
		_server = nil;
		[self didChangeValueForKey:@"server"];
	}
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// object is always _controller and keyPath = @"server"
	id server = [_controller server];
	if(server && _server == nil){ // get connected after being disconnected
		[self willChangeValueForKey:@"server"];
		@try{
			_server = [[(id)[_controller server]getStockAdmin] retain];
			[_server setDelegate:self];
		}@catch(NSException *localException){
			NSLog(@"%@, %@", [localException name], [localException reason]);
		}
		[self didChangeValueForKey:@"server"];
	}
}

@end
