/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "Tree.h"


@implementation Tree

- initName:(NSString*)name box:box
{
    self = [super init];
    _name = [name retain];
    _box = [box retain];
    return self;
}

- init
{
    return [self initName:@"tree" box:nil];
}

- (void) dealloc
{
    [_name release];
    [_box release];
    [_left release];
    [_right release];
	[super dealloc];
}

- (void) setUp:up
{
    _up = up;
}

- (void) setLeft:left
{
    if(_left != left){
        [_left autorelease];
        _left = [left retain];
        if([_left respondsToSelector:@selector(setUp:)])
            [_left setUp:self];
    }
}

- (void) setRight:right
{
    if(_right != right){
        [_right autorelease];
        _right = [right retain];
        if([_right respondsToSelector:@selector(setUp:)])
            [_right setUp:self];
    }
}

- (NSString*) description:(NSMutableArray*)array
{
    NSString* keys[] = {@"up",@"left", @"right",@"box",nil};
    int j;
    NSMutableString* str = [NSMutableString stringWithFormat:@"%@ 0x%x %@{\n", [self className], self, _name];
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    [array addObject:self];
    for(j=0;keys[j] != nil; j++){
        id obj = [self valueForKey:keys[j]];
        if(obj == nil)
            [str appendFormat:@"%@ = <nil>\n", keys[j]];
        else
        if([array containsObject:obj])
            [str appendFormat:@"%@ = 0x%x\n", keys[j], obj];
        else{
            [array addObject:obj];
            [str appendFormat:@"%@ = %@\n", keys[j],
                [obj respondsToSelector:_cmd]?[obj description:array]:[obj description]];
        }
    }
    [str appendString:@"}"];
    [pool release];
    return str;
}

- (NSString*) description
{
    return [self description:[NSMutableArray array]];
}

@end
