/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "ValueTestServer.h"
#import "Box.h"
#import "Tree.h"
#import "WTree.h"
#import "NamingContextProto.h"

@implementation ValueTestServer

static NSString* interfaceName = @"vtest/ValueTest";

// Provide this method so that an instance can be activated
- (IDLInterface*) interface
{
    return [[ORB defaultORB]interfaceNamed:interfaceName];
}

- init
{
    self = [super init];
    [WTree class]; // force to initialize
	_link = [[IDLObject objectWithIDLType:[[ORB defaultORB]definitionNamed:@"vtest/AccessBox"]]retain];
	_node = [[IDLObject objectWithIDLType:[[ORB defaultORB]definitionNamed:@"vtest/Link"]]retain];
	[_link setValue:@"link" forKey:@"name"];
	[_link setValue:self forKey:@"link"];
	[_node setValue:@"node" forKey:@"name"];
	[_node setValue:[NSNumber numberWithDouble:1.0] forKey:@"weight"];
    return self;
}

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
    [super dealloc];
}

- (void) run
{
    NSString* ior;
    NSString* path = [[NSFileManager defaultManager]currentDirectoryPath];
    CORBAObject* nameService = [ORB resolveInitialReferences:@"NameService"];
    if(nameService)
        // Rebind self in the Naming Service. 'self' will be implicitly activated
        [nameService rebind:[IDLNameComponent nameComponentsForPath:@"ORBValueTest"] object:self];
    else
        // Otherwise we have to activate 'self' explicitly
        [self activate];
    // Save IOR to the file
    ior = [[self objectReference]toString];
    if([[[self objectReference]toString] writeToFile:[path stringByAppendingPathComponent:@"vtest.ref"]
										  atomically:NO encoding:NSUTF8StringEncoding error:NULL]){
        // Just a test. 
        if([CORBAObject objectWithIOR:ior] != [self objectReference])
            NSLog(@"NOT WORKING !!!");
        NSLog(@"Saved IOR in './vtest.ref'\n===> ORB is running ...\n");
        // Run ORB
        [ORB run];
    }else
        NSLog(@"Cannot write IOR to file './ORBTest.ref'\n");
    
}

#pragma mark --- Implementation methods ---

- getBox:(NSString*)name width:(double)w height:(double) h
{
    return [[[Box alloc]initName:name width:w height:h]autorelease];
}

- getNode:(NSString*) name
{
// valuetype Tree has associated ObjC class
    return [[[Tree alloc]initName:name box:nil]autorelease];
}

- getLink:(NSString*)name weight:(double)weight
{
    // IDLValueType cannot provide valuetype for NSDictionary and 'Node' would be used
    // We set an aditional entry so that our observer will provide the correct vauetype
    id value = [[ORB defaultORB]definitionNamed:@"vtest/Link"];
    return [NSDictionary dictionaryWithObjectsAndKeys:
        name?name:@"", @"name",
        [NSNumber numberWithDouble:weight], @"weight",
        value, @"valuetype", nil];
}

- getTree
{
    id box = [[[Box alloc]initName:@"box0" width:5.0 height:7.0]autorelease];
    id root = [[[Tree alloc]initName:@"root" box:box]autorelease];
    id left = [[[WTree alloc]initName:@"left0" 
        box:[[[Box alloc]initName:@"boxl1" width:25.0 height:73.0]autorelease]]autorelease];
    id right = [[[WTree alloc]initName:@"right0" 
        box:[[[Box alloc]initName:@"boxr1" width:33.0 height:44.0]autorelease]]autorelease];
    
    id common = [[[Tree alloc]initName:@"common" box:box]autorelease];
    
    [root setLeft:left];
    [root setRight:right];
    [left setRight:common];
    [right setLeft:common];
    [left setValue:[NSNumber numberWithDouble:2.5] forKey:@"weight"];
    [right setValue:[NSNumber numberWithDouble:3.5] forKey:@"weight"];
    [common setRight:root];
    [common setLeft:root];
    return root;
}

- (BOOL) getMulti:(out id*)value boxes:(out NSArray**)boxes
{
    *value = [self getTree];
    *boxes = [NSArray arrayWithObjects:
        [[[Box alloc]initName:@"box0" width:5.0 height:7.0]autorelease],
        [[[Box alloc]initName:@"box1" width:7.0 height:9.0]autorelease],
        [[[Box alloc]initName:@"box2" width:9.0 height:11.0]autorelease],
        [[[Box alloc]initName:@"box3" width:11.0 height:5.0]autorelease],
        nil];
    return YES;
}

- echoNode:node
{
    return node;
}

- getAccess
{
// return interface
    return self;
}

- getObjectAccess
{
// return valuetype
    id box = [[[Box alloc]initName:@"box0" width:5.0 height:7.0]autorelease];
    return [[[Tree alloc]initName:@"root" box:box]autorelease];
}

// valuetype
- getAny1
{
    IDLDefinition* def = [[ORB defaultORB]definitionNamed:@"vtest/Tree"];
	IDLAnyObject* any = [IDLAnyObject objectWithIDLType:def];
	[any setValue:[self getTree]];
	return any;
}
// 
- getAny2
{
    IDLDefinition* def = [[ORB defaultORB]definitionNamed:@"vtest/ValueTest"];
	IDLAnyObject* any = [IDLAnyObject objectWithIDLType:def];
	[any setValue:self];
	return any;
}

- getAny3
{
    IDLDefinition* def = [[ORB defaultORB]definitionNamed:@"vtest/Access"];
	IDLAnyObject* any = [IDLAnyObject objectWithIDLType:def];
	[any setValue:[self getObjectAccess]];
	return any;
}

- getAnyBox
{
    IDLDefinition* def = [[ORB defaultORB]definitionNamed:@"vtest/VBox"];
	IDLAnyObject* any = [IDLAnyObject objectWithIDLType:def];
	[any setValue:[self getBox:@"box25" width:3 height:4]];
	return any;
}

- echoAny:value
{
    return value;
}

- (NSString*) getName
{
    return @"ValueTest";
}

- (double) getWeight
{
    return 0.0;
}

- (NSNumber*) echoLong:(NSNumber*) value
{
    NSLog(@"%@: %@ - %@", NSStringFromSelector(_cmd), NSStringFromClass([value class]), value);
    return value;
}

- (NSNumber*) echoDouble:(NSNumber*) value
{
    NSLog(@"%@: %@ - %@", NSStringFromSelector(_cmd), NSStringFromClass([value class]), value);
    return value;
}
- (NSNumber*) echoBool:(NSNumber*) value
{
    NSLog(@"%@: %@ - %@", NSStringFromSelector(_cmd), NSStringFromClass([value class]), value);
    return value;
}
- (NSNumber*) echoChar:(NSNumber*) value
{
    NSLog(@"%@: %@ - %@", NSStringFromSelector(_cmd), NSStringFromClass([value class]), value);
    return value;
}

- (NSString*) echoString:(NSString*) value
{
    NSLog(@"%@: %@ - %@", NSStringFromSelector(_cmd), NSStringFromClass([value class]), value);
    return value;
}

- (NSArray*) echoLongSeq:(NSArray*)value
{
    NSLog(@"%@: %@ - %@", NSStringFromSelector(_cmd), NSStringFromClass([value class]), [value description]);
    return value;
}

- echoAbstract:value
{
    NSLog(@"%@: %@ - %@", NSStringFromSelector(_cmd), NSStringFromClass([value class]), [value description]);
    return value;
}

- echoValue:value
{
    return value;
}

- echoBox:value
{
    return value;
}

- _getLink
{
	return _link;
}
- (void) _setLink:link
{
	[_link release];
	_link = [link retain];
}
- _getNode
{
	return _node;
}

- (void) _setNode:node
{
	[_node release];
	_node = [node retain];

}

int main (int argc, const char *argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    id test = [[ValueTestServer alloc]init];
    [test run];
    [test release];
    [pool release];
    exit(0);       // insure the process exit status is 0
    return 0;      // ...and make main fit the ANSI spec.
}

@end

