/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLFunction_Private.h"
#import "IDLArgument.h"
#import <Foundation/Foundation.h>

@implementation IDLFunction(Selectors)

static void convertBuffer(char* buf);

- (NSString*) _selectorName
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
	char buf[1024];
	char* b = buf, *b0 = buf;
	[_name getCString:buf maxLength:1024 encoding:NSUTF8StringEncoding];
	b = b + strlen(b);
	NSEnumerator* en = [_args objectEnumerator];
	NSString* argName = [[en nextObject]name];
	if(argName)
		*b++ = ':';
	while(argName = [[en nextObject]name]){
		[argName getCString:b maxLength:1024-(b-b0) encoding:NSUTF8StringEncoding];
		b += strlen(b);
		*b++ = ':';
	}
	if(_contextList){
		if([_args count] > 0)
			strcpy(b,"context:");
		else
			strcpy(b, ":");
	}else
		*b++ = 0;
	convertBuffer(buf);
	[pool release];
	return [NSString stringWithUTF8String:buf];
}

- (NSString*) selectorName
{
    if(!_selector)
        _selector = [[self _selectorName]retain];
    return _selector;
}

- (void) setSelector:(SEL)selector
{
	if(selector){
		NSString *selName = NSStringFromSelector(selector);
		[self setSelectorName:selName];
	}
}

- (void) setSelectorName:(NSString*)selName
{
	if(selName && ![_selector isEqualToString:selName]){
		int numArgs = [_args count];
		if(_contextList)
			numArgs += 1;
		const char *b = [selName cStringUsingEncoding:NSASCIIStringEncoding];
		if(!b)
			[NSException raise:NSInternalInconsistencyException 
						format:@"selector name must contain only ASCII characters, method '%@', selector '%@'", _name, selName];
		int n = 0;
		while(b = strchr(b+1,':')) n += 1;
		if(n != numArgs)
			[NSException raise:NSInternalInconsistencyException
						format:@"invalid selector for method '%@', number of arguments must be %d", _name, numArgs];
		NSString *currentSelector = _selector;
		[_selector release];
		_selector = [selName copy];
		if(currentSelector) // reset interface
			[_interfaceObject resetSelectors];
	}
}

@end

static int to_upper(int c)
{
    if(c>='a' && c<='z')
        return c + 'A' - 'a';
    else
        return c;
}
static void convertBuffer(char* buf)
{
    char* b = ++buf; // Skip first char
    while(*buf){
        switch(*buf){
            default:
                if(b != buf) *b = *buf;
                b += 1;
                buf += 1;
                break;
            case '&':
                buf += 1;
                break;
            case '_':
                buf += 1;
                if(*buf){
                    *b++ = to_upper(*buf);
                    buf += 1;
                }else
                    *b++ = buf[-1]; //last char
                break;
        }
    }
    *b = 0;
}

