/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLServiceContext.h"
#import "IDLEncapsulation.h"
#import "IDLDefinitionScope.h"
#import "IDLDefinition_Private.h"
#import "CORBAObject.h"
#import "IDLIOPProfile.h"
#import "IDLTaggedComponent.h"
#import "ORB_Private.h"

/* 
 * Codesets
 *
 * NSISOLatin1StringEncoding		= 0x00010001	- ISO8859_1
 * NSISOLatin2StringEncoding		= 0x00010002	- ISO8859_2
 * kCFStringEncodingISOLatin3		= 0x00010003	- ISO8859_3
 * kCFStringEncodingISOLatin4		= 0x00010004	- ISO8859_4
 * kCFStringEncodingISOLatinCyrillic = 0x00010005   - ISO8859_5
 * kCFStringEncodingISOLatinArabic  = 0x00010006	- ISO8859_6
 * kCFStringEncodingISOLatinGreek   = 0x00010007	- ISO8859_7
 * kCFStringEncodingISOLatinHebrew  = 0x00010008	- ISO8859_8
 * kCFStringEncodingISOLatin5		= 0x00010009	- ISO8859_9
 * kCFStringEncodingISOLatin9		= 0x0001000F	- ISO8859_15_FDIS
 * NSWindowsCP1250StringEncoding	= 0x100204E2	- Cp1250
 * NSWindowsCP1251StringEncoding	= 0x100204E3	- Cp1251
 * NSWindowsCP1252StringEncoding	= 0x100204E4	- Cp1252
 * NSWindowsCP1253StringEncoding	= 0x100204E5	- Cp1253
 * NSWindowsCP1254StringEncoding	= 0x100204E6	- Cp1254
 * kCFStringEncodingWindowsHebrew   = 0x100204E7	- Cp1255
 * kCFStringEncodingWindowsArabic   = 0x100204E8	- Cp1256
 * kCFStringEncodingWindowsBalticRim = 0x100204E9   - Cp1257
 * kCFStringEncodingDOSKorean		= 0x100203B5	- Cp949C
 * kCFStringEncodingDOSChineseTrad  = 0x100203B6	- Cp950
 * NSJapaneseEUCStringEncoding		= 0x00030010	- EUC_JP
 * kCFStringEncodingEUC_KR			= 0x0004000A	- EUC_KR
 * kCFStringEncodingEUC_TW			= 0x00050010	- EUC_TW
 * NSUnicodeStringEncoding			= 0x00010100	- UCS2 ?
 * NSUTF8StringEncoding				= 0x05010001	- UTF8
 * NSUnicodeStringEncoding			= 0x00010109	- UTF16
 * ? = 0x100203AF   - Cp943C
 * ? = 0x10020553   - Cp1363C
 * ? = 0x100203C4   - Cp964
 * ? = 0x100203CA   - Cp970
 * ? = 0x10020567   - Cp1383
 * ? = 0x100283BA   - Cp33722C
 * ? = 0x100203A2   - Cp930
 * ? = 0x10020417   - Cp1047
 */

@implementation IDLServiceContext

static NSDictionary* _stringCodes = nil;
static NSDictionary* _wstringCodes = nil;
static NSDictionary* _rstringCodes = nil;
static NSDictionary* _rwstringCodes = nil;
static NSArray* _stringCodesArray = nil;
static NSArray* _wstringCodesArray = nil;

static NSMutableDictionary* _contextPaths = nil;

NSString* ORBDefaultStringEncodingKey = @"ORBDefaultStringEncoding";
NSString* ORBDefaultWStringEncodingKey = @"ORBDefaultWStringEncoding";

+ (void) _setupStringEncodings
{
	int code[] = {
		NSUTF8StringEncoding, 0x05010001,
		NSASCIIStringEncoding, 0x00010020,
		NSISOLatin1StringEncoding, 0x00010001,
		NSISOLatin2StringEncoding, 0x00010002,
		kCFStringEncodingISOLatin3, 0x00010003,
		kCFStringEncodingISOLatin4, 0x00010004,
		kCFStringEncodingISOLatinCyrillic, 0x00010005,
		kCFStringEncodingISOLatinArabic, 0x00010006,
		kCFStringEncodingISOLatinGreek, 0x00010007,
		kCFStringEncodingISOLatinHebrew, 0x00010008,
		kCFStringEncodingISOLatin5, 0x00010009,
		kCFStringEncodingISOLatin9, 0x0001000F,
		NSWindowsCP1250StringEncoding, 0x100204E2,
		NSWindowsCP1251StringEncoding, 0x100204E3,
		NSWindowsCP1252StringEncoding, 0x100204E4,
		NSWindowsCP1253StringEncoding, 0x100204E5,
		NSWindowsCP1254StringEncoding, 0x100204E6,
		kCFStringEncodingWindowsHebrew, 0x100204E7,
		kCFStringEncodingWindowsArabic, 0x100204E8,
		kCFStringEncodingWindowsBalticRim, 0x100204E9,
		kCFStringEncodingDOSKorean, 0x100203B5,
		kCFStringEncodingDOSChineseTrad, 0x100203B6,
		NSJapaneseEUCStringEncoding, 0x00030010,
		kCFStringEncodingEUC_KR, 0x0004000A,
		kCFStringEncodingEUC_TW, 0x00050010,
		0
	};
	int wcode[] = {
		NSUnicodeStringEncoding, 0x00010100,
		NSUnicodeStringEncoding, 0x00010109,
		0
	};
	
	NSMutableDictionary* dict = [NSMutableDictionary dictionary];
	NSMutableDictionary* rdict = [NSMutableDictionary dictionary];
	NSMutableArray* array = [NSMutableArray array];
	int n;
	for(n=0; code[n] != 0; n+=2){
		[dict setObject:[NSNumber numberWithUnsignedInt:code[n]] forKey:[NSNumber numberWithUnsignedInt:code[n+1]]];
		[rdict setObject:[NSNumber numberWithUnsignedInt:code[n+1]] forKey:[NSNumber numberWithUnsignedInt:code[n]]];
		[array addObject:[NSNumber numberWithUnsignedInt:code[n+1]]];
	}
	_stringCodes = [[NSDictionary alloc]initWithDictionary:dict];
	_rstringCodes = [[NSDictionary alloc]initWithDictionary:rdict];
	_stringCodesArray = [[NSArray alloc]initWithArray:array];
	[dict removeAllObjects];
	[array removeAllObjects];
	[rdict removeAllObjects];
	for(n=0; wcode[n] != 0; n+=2){
		[dict setObject:[NSNumber numberWithUnsignedInt:wcode[n]] forKey:[NSNumber numberWithUnsignedInt:wcode[n+1]]];
		[rdict setObject:[NSNumber numberWithUnsignedInt:wcode[n+1]] forKey:[NSNumber numberWithUnsignedInt:wcode[n]]];
		[array addObject:[NSNumber numberWithUnsignedInt:wcode[n+1]]];
	}
	_wstringCodes = [[NSDictionary alloc]initWithDictionary:dict];
	_rwstringCodes = [[NSDictionary alloc]initWithDictionary:rdict];
	_wstringCodesArray = [[NSArray alloc]initWithArray:array];
}

+ (void) registerContextId:(unsigned)ctxtId withIDLPath:(NSString*)path
{
	NSNumber* key = [NSNumber numberWithUnsignedInt:ctxtId];
	if(path)
		[_contextPaths setObject:path forKey:key];
	else
		[_contextPaths removeObjectForKey:key];
}

+ (void) initialize
{
	if([IDLServiceContext class] == self){
		_contextPaths = [[NSMutableDictionary alloc]init];
		[self registerContextId:ORB_CodeSets withIDLPath:@"CONV_FRAME/CodeSetContext"];
		[self registerContextId:ORB_BI_DIR_IIOP withIDLPath:@"IIOP/BiDirIIOPServiceContext"];
		[self _setupStringEncodings];
		NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithUnsignedInt:0], ORBDefaultStringEncodingKey, // don't send codesets context
			[NSNumber numberWithUnsignedInt:NSUnicodeStringEncoding], ORBDefaultWStringEncodingKey,
			nil];
		[[NSUserDefaults standardUserDefaults]registerDefaults:dict];
	}
}

+ (NSDictionary*) currentContextDictionary
{
	NSDictionary* dict = [[[NSThread currentThread]threadDictionary]objectForKey:@"serviceContext"];
	return dict? [NSDictionary dictionaryWithDictionary:dict] : nil;
}

+ (IDLServiceContext*) currentContextForContextId:(unsigned)contextId
{
	return [[[[NSThread currentThread]threadDictionary]objectForKey:@"serviceContext"]objectForKey:[NSNumber numberWithUnsignedInt:contextId]];
}

+ (void) setCurrentServiceContext:(NSDictionary*)ctxt
{
	[[[NSThread currentThread]threadDictionary]setObject:ctxt forKey:@"serviceContext"];
}

+ (void) addContext:(IDLServiceContext*)context
{
	NSMutableDictionary* threadDict = [[NSThread currentThread]threadDictionary];
	NSMutableDictionary* dict = [threadDict objectForKey:@"serviceContext"];
	NSNumber* key = [NSNumber numberWithUnsignedInt:[context contextId]];
	if(dict)
		[dict setObject:context forKey:key];
	else
		[threadDict setObject:dict = [NSMutableDictionary dictionaryWithObject:context forKey:key] forKey:@"serviceContext"];
}

+ (void) removeContextWithContextId:(unsigned)contextId
{
	NSMutableDictionary* threadDict = [[NSThread currentThread]threadDictionary];
	NSMutableDictionary* dict = [threadDict objectForKey:@"serviceContext"];
	NSNumber* key = [NSNumber numberWithUnsignedInt:contextId];
	[dict removeObjectForKey:key];
}

+ (NSStringEncoding) stringEncodingWithCode:(unsigned)code
{
	NSNumber* encoding = [_stringCodes objectForKey:[NSNumber numberWithUnsignedInt:code]];
	return encoding? [encoding unsignedIntValue] : NSNotFound;
}
+ (NSStringEncoding) wstringEncodingWithCode:(unsigned)code
{
	NSNumber* encoding = [_wstringCodes objectForKey:[NSNumber numberWithUnsignedInt:code]];
	return encoding? [encoding unsignedIntValue] : NSNotFound;
}

- (void) _encodeWithDefPath:(NSString*)path
{
    IDLDefinition* def = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionForNamePath:path];
	if(!def)
		def = (IDLDefinition*)[[[ORB defaultORB] rootScope]definitionForNamePath:path];
    if(def){
        IDLEncapsulation* stream = [IDLEncapsulation encapsulation];
        [def encodeObject:_context withCoder:stream];
        [_context_data release];
        _context_data = [[stream data]retain];
    }else{
		NSLog(@"IDLServiceContext: cannot find definition for  IDL path %@", path);
        [NSException raise:CORBADefinitionException format:@"Cannot find definition for path '%@'", path];
	}
}

- (void) encode
{
	NSString* path = [_contextPaths objectForKey:[NSNumber numberWithUnsignedInt:_context_id]];
	if(path)
		[self _encodeWithDefPath:path];
	else if(!_context_data){
		NSLog(@"IDLServiceContext: no IDLPath is registered for service contextId %u", _context_id);
		[NSException raise:CORBADefinitionException format:@"no IDLPath is registered for service contextId %u", _context_id];
	}
}

- initWithContextId:(unsigned)tag context:context
{
    self = [super init];
    _context_id = tag;
    _context = [context retain];
//    [self encode];
    return self;
}

// Deprecated method
- initWithTag:(unsigned)tag context:context;
{
	return [self initWithContextId:tag context:context];
}

+ serviceContextWithContextId:(unsigned)tag context:context;
{
	return [[[self alloc]initWithContextId:tag context:context]autorelease];
}

+ codeSetsServiceContext:(unsigned)charData:(unsigned)wcharData
{
    NSDictionary* value = [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithUnsignedInt:charData], @"char_data",
        [NSNumber numberWithUnsignedInt:wcharData], @"wchar_data", nil];
    return [[[self alloc]initWithContextId:ORB_CodeSets context:value]autorelease];
}

- init
{
    return [self initWithContextId:0 context:nil];
}

- (void) dealloc
{
    [_context_data release];
    [_context release];
    [super dealloc];
}

- (void) _decodeWithDefPath:(NSString*)path
{
    IDLDefinition* def = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionForNamePath:path];
	if(!def)
		def = (IDLDefinition*)[[[ORB defaultORB] rootScope]definitionForNamePath:path];
    if(def){
        IDLEncapsulation* stream = [IDLEncapsulation encapsulationWithData:(NSMutableData*)_context_data];
        [def decodeObject:&_context withCoder:stream];
        [_context retain];
    }else
        [NSException raise:CORBADefinitionException
                    format:@"Cannot find definition for '%@'", path];
}

- (void) decode
{
	NSString* path = [_contextPaths objectForKey:[NSNumber numberWithUnsignedInt:_context_id]];
	if(path)
		[self _decodeWithDefPath:path];
}

- (void) _setContext_data:(NSData*)data
{
    if(data != _context_data){
        [_context_data release];
        _context_data = [data retain];
        [self decode];
    }
}

- (unsigned) contextId
{
	return _context_id;
}

- context_data
{
	if(_context)
		[self encode];
	return _context_data;
}

- context
{
	return _context;
}

+ codeSetsWithStringEncodings:(NSStringEncoding)stringEnc :(NSStringEncoding)wstringEnc
{
	NSNumber* charCode = [_rstringCodes objectForKey:[NSNumber numberWithUnsignedInt:stringEnc]];
	NSNumber* wcharCode = [_rwstringCodes objectForKey:[NSNumber numberWithUnsignedInt:wstringEnc]];
    NSDictionary* value = [NSDictionary dictionaryWithObjectsAndKeys:
        charCode, @"char_data", wcharCode, @"wchar_data", nil];
    return [[[self alloc]initWithContextId:ORB_CodeSets context:value]autorelease];
}


- (NSString*) description
{
	return [NSString stringWithFormat:@"<%@ 0x%x contextId=%d>", NSStringFromClass([self class]), self, _context_id];
}

+ (NSStringEncoding) defaultStringEncoding
{
	return [[NSUserDefaults standardUserDefaults]integerForKey:ORBDefaultStringEncodingKey];
}

+ (NSStringEncoding) defaultWStringEncoding
{
	return [[NSUserDefaults standardUserDefaults]integerForKey:ORBDefaultWStringEncodingKey];
}

+ (void) setDefaultStringEncoding:(NSStringEncoding)encoding
{
	[[NSUserDefaults standardUserDefaults]setInteger:encoding forKey:ORBDefaultStringEncodingKey];
}

+ (void) setDefaultWStringEncoding:(NSStringEncoding)encoding
{
	[[NSUserDefaults standardUserDefaults]setInteger:encoding forKey:ORBDefaultWStringEncodingKey];
}

@end
